;Source code file for Motorola C-205 lcd interface: 
; LCD = GPM-308B0
; 
; internal clock
; standard crystal 4.000 MHz XT - 1us pe instructiune
; Program realizat de Ing. Bergthaller Iulian-Alexandru 
;------------------------------------------------------------
  list  p=16f84A;f=inhx8m
_CP_OFF equ H'3FFF'       ;code protect off
_PWRTE_ON equ H'3FFF'     ;Power on timer on
_WDT_OFF  equ H'3FFB'   ;watch dog timer off
_XT_OSC   equ H'3FFD' ;crystal oscillator
  __CONFIG       _CP_OFF & _PWRTE_ON & _WDT_OFF & _XT_OSC
;------------------------------------------------------------
;      cpu init
status  equ 03
porta       equ 05
portb       equ 06
pcl     equ 02
fsr     equ 04
esp     equ 30
cifra   equ 31
indf    equ 00
tmr     equ 01

count1  equ 14
count2  equ 15
count3  equ 16
transo  equ 17
r2      equ 18
r1      equ 19

fpch    equ 20
freqH   equ 21
freqM   equ 22
freqL   equ 23
fget    equ 24
divH    equ 25
divL    equ 26
fstep   equ 27
control equ 28
band    equ 29 
r3      equ 32
r4      equ 33

trisa   equ 0x5
trisb   equ 0x6  

#DEFINE sclk  porta,1
#DEFINE sdta  porta,0
#DEFINE rst   portb,7
#DEFINE sclks  porta,2
#DEFINE sdtas  porta,3
#DEFINE weakbus  portb,0
#DEFINE f_inc  portb,1
#DEFINE f_dec  portb,2

;------------------------------------------------------------
;      bit init
rp0 equ 5
w equ 0
f equ 1
;------------------------------------------------------------
  org 0
;
;------------------------------------------------------------
init  bsf status,rp0  ;switch to bank 1
  movlw 0x0 ; porta and portb set to outputs
  movwf trisa  ; switch to out mode 
  movlw 0x7F
  movwf trisb   ; switch to input mode b0-b6, out mode b7
  bcf status,rp0  ;switch back to bank 0
  movlw 0x00
  movwf porta  ;  clear porta
  movwf portb  ;  clear portb
;***************************************************
start 
;**************** lcd reset ******************
  bcf   rst
  bsf   sclk
  bsf   sdta
  call  demorst
  goto  label2
;
;*** pause 200 ms ***************
;
demorst  movlw 0xFF
  movwf   r2
dersto2  movlw 0xFF
  movwf   r1
dersto1  decfsz r1,f
  goto    dersto1
  decfsz  r2,f
  goto    dersto2
  return
;
;********************
label2  bsf  rst
;
;*** end lcd-reset ***************
;

;***************************************************
;* Lcd initialisation                               *
;***************************************************
scrie call  iict  ; i2c start
  movlw 0x78  ; device write address (74h for read)
  call  act ; i2c send subroutine
  movlw 0x00  ; Co,D/C=0, set instruction register for multiple next command bytes  
  call  act
  movlw 0x31  ; 
  call  act
  movlw 0x14  ; lcd on
  call  act
  movlw 0x06  ; direction r>l
  call  act
  movlw 0x30  ; clears entire display 
  call  act
  movlw 0x11  ; address=0
  call  act
  movlw 0x05  ; end of linia2
  call  act
  movlw 0x31
  call  act
  movlw 0x9A
  call  act
  movlw 0x0C
  call  act
  movlw 0x30
  call  act
  movlw 0x0C
  call  act
  movlw 0x40    ; Y
  call  act
  movlw 0x80    ; X
  call  act
  call  iicp  ; i2c stop 
;
;********* clear RAM lcd *************----------
; 
  call  iict
  movlw 0x78
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x66
  movwf   r2
dersto7  movlw 0x09
  movwf   r1
dersto8  decfsz r1,f
  goto  fetr1
  goto  fetr
fetr1  movlw 0x00  ;
  call  act
  goto    dersto8
fetr  decfsz  r2,f
  goto    dersto7
  call    iicp
; ----------------------- end clear RAM lcd -----
   
   ;----------------------- set frequency registers -------------
     
     movlw  0x50         ;  frequency step in kHz
     movwf  fstep
     movlw  0x0B
     movwf  divH        ; byte divider high
     movlw  0x5E
     movwf  divL        ; byte divider low
     movlw  0xC0      
     movwf  control     ; control byte
     movlw  0x01
     movwf  band        
     movlw  0x10
     movwf  freqH
     movlw  0x75
     movwf  freqM
     movlw  0x00
     movwf  freqL
     call   ksh
   
   call  lcd_out
   call  space1
 
   call  mhz
;
; -------------- start opros klavi --------------- 

  clrf  r3
  bcf  esp,2      ; flag for freq_inc
  bcf  esp,3      ; flag for freq_dec

opros  btfss   weakbus
      call   weakb
;
           btfss  f_inc
           goto  opr21        ; f_inc=0 -> nazata
           goto  opr31        ; f_inc=1 -> ne nazata
opr21       bcf   esp,3
            movf  r3,w
            bcf   status,2
            xorlw  0x05
            btfss status,2    ; z=0 ?
            goto  opr71       ; no
            bsf  esp,2        ; yes
            goto  opr41
opr71       incf  r3,1     
opr41       call  freq_inc
            goto  opr51
opr31        btfsc  esp,2     ; esp,2=0 ?
             goto   opr41     ; no
             clrf   r3        ; yes
opr51      
;
       btfss   f_dec
           goto  opr22        ; f_inc=0 -> nazata
           goto  opr32        ; f_inc=1 -> ne nazata
opr22       bcf   esp,2
            movf  r4,w
            bcf   status,2
            xorlw  0x05
            btfss status,2    ; z=0 ?
            goto  opr72       ; no
            bsf  esp,3        ; yes
            goto  opr42
opr72       incf  r4,1
opr42       call  freq_dec
            goto  opr52
opr32        btfsc  esp,3     ; esp,2=0 ?
             goto   opr42     ; no
             clrf   r4        ; yes
opr52      
      
 ;    btfsc  esp,2
  ;   goto   opr60
  ;   btfss  esp,3
  ;   call pause1
;opr60  call pause
  ;
  goto  opros

;--------- end opros klavi -----------------------    
 goto konec
;
;------------------start  LCD  --------------------
lcd_out   ;
    call  iict
  movlw 0x78
  call  act
  movlw 0x00  ; Co=0, RS=1, set instruction register for multiple next data bytes  
  call  act
  movlw 0x30  ; 
  call  act
  movlw 0x0C  ; display config=LSB on top
  call  act
  movlw 0x43  ; address Y=
  call  act
  movlw 0x90  ; address X=
  call  act
  call  iicp
;
 
;
      movlw   freqH    ; write const=R1 into reg w
        movwf   fsr   ; load from w=R1 into reg fsr
        bcf     esp,0   ; erase 0bit of reg esp
        clrf    cifra
        bcf     esp,1   ; erase 1bit of reg esp
u0      movf    indf,0    ; load W=indf ; Z=endif==0
        andlw   h'f0'    ; W=W & h'f0' ; Z=(W&h'f0')==0 
;
        btfss   status,2
        goto    u1
       
uu2  movwf    f
        swapf    f,1
        movlw    h'0f'
        andwf    f,0
nule  call   cikl_2
;--- print dot
        movlw   .2
        xorwf   cifra,0
        btfsc   status,2
        call dot               ; print on lcd "dot"

dotend  incf   cifra,1
;--- 
        btfsc   esp,1         ; 䫠  1
        goto    u2
        bsf     esp,1
        swapf   indf,1
        goto    u0

u1      bsf     esp,0         ; 䫠
        goto    uu2
u2      bcf     esp,1
        swapf   indf,1
        incf    fsr,1
        movlw   freqL+1
        xorwf   fsr,0
        
        btfss   status,2
        goto    u0

        bcf     esp,0         ; 䫠
        
        retlw   00
;----------------------end LCD-------------
;
;  ------ subroutine digits--------------
cikl_2   ;movf  r1,w
   addwf   pcl,F
 goto   dig0
 goto   dig1
 goto   dig2
 goto   dig3
 goto   dig4
 goto   dig5
 goto   dig6
 goto   dig7
 goto   dig8
 goto   dig9
cikl_3  ;decfsz  r1,F
 ; goto   cikl_2
; 
 retlw  00
;--------------------
;
;
;---------------------------------------------------
;
;
;----------- start set ks-h-148 ------------
ksh     
      ;
 call  iicts      ; i2c start
  movlw 0xC0      ; device address write mode (C1h for read mode)
  call  acts      ;    i2c send subroutine
  movf  divH,w    ;    divider 1
  call  acts
  movf  divL,w    ;    divider 2
  call  acts
  movf  control,w ;   control byte
  call  acts
  movf  band,w    ;  band switch byte
  call  acts
 call  iicps      ; i2c stop
  
 retlw  00
;   ------------  end set ks-h-148 ------------
;
;  ----------------start set weak booster---------------- 
weakb   btfsc   control,0   ;   set weak booster
      goto   weakb1
      bsf    control,0
       call  weakb_on
      goto   weakb2
weakb1   bcf    control,0
     call  weakb_off
weakb2   call   ksh

    call pause1
    retlw 00                  ;
; -------------------end set weak booster-------------------
;
; ----------------start increment frequency  -----
freq_inc   incfsz   divL,1
         goto   ksh11
         incf   divH,1
ksh11     
          movf  fstep,w

          addwf freqL,1
          movlw   0xA0
          bcf   status,2
          xorwf  freqL,w
          btfss   status,2        ; 
          goto  ksh21             ;
          movlw   0x00
          movwf   freqL
           incf  freqM,1
           movf   freqM,w
           andlw   0x0F
           bcf   status,2
           xorlw  0x0A
           btfss   status,2        ; 
           goto   ksh21
          movlw   0xF0
          andwf   freqM,1
          movlw   0x10
          addwf   freqM,1
ksh51     movf   freqM,w
          andlw   0xF0
          bcf   status,2          ; z=0
          xorlw   0xA0            ; w=w^0xA0
          btfss   status,2        ; z=1 ?
          goto     ksh21
          movlw   0x0F
          andwf   freqM,1         ; 4 high bytes freqM = 0
          ; bcf   status,2
           incf   freqH,1
           movf   freqH,w
           andlw   0x0F
           bcf   status,2
           xorlw  0x0A
           btfss   status,2        ; 
           goto   ksh21
           
          movlw   0xF0
          andwf   freqH,1
          movlw   0x10
          addwf   freqH,1    

ksh21   bcf   status,2      ; detect band switching
        ; movf    divL,w
         movlw  0x60
          xorwf  divL,w          
         btfss   status,2 
         goto   ksh31
        bcf   status,2
         movlw  0x0E
          xorwf  divH,w
         btfss   status,2
         goto   ksh41
         bcf   band,0
         bsf   band,1
         bcf   band,2
ksh41     bcf   status,2
         movlw  0x22
          xorwf  divH,w
         btfss   status,2
         goto   ksh31
         bcf   band,0
         bcf   band,1
         bsf   band,2
ksh31     bcf status,2  ; to 
          movlw  0x21
          xorwf  divL,w
          btfss  status,2
          goto   ksh71
          bcf  status,2
          movlw   0x4E
          xorwf  divH,w
          btfss  status,2
          goto   ksh71
          movlw  0x18
          movwf  divL
          movlw  0x06
          movwf  divH
          movlw  0x04
          movwf  freqH
          movlw  0x00
          movwf  freqM
          movlw  0x00
          movwf  freqL
ksh71       call   ksh
          call  lcd_out
     btfsc  esp,2
     goto   opra60
     call pause1
opra60  call pause
        ;call   pause1
       ;bsf  esp,1
      retlw  00
; ---------end increment frequency------------------
;
;---------start decrement frequency  -----------
freq_dec   decfsz   divL,1
         goto   ksh12
         decf   divH,1
ksh12     ; call   ksh
          movf  fstep,w
          ;bcf   status,2
        subwf freqL,1
          bcf   status,2
          movlw   0xB0
          xorwf  freqL,w
         btfss  status,2
          goto   ksh22
          movlw   0x50
          movwf   freqL
           decf freqM,1
           movf  freqM,w
           andlw  0x0F
           bcf   status,2
           xorlw  0x0F
           btfss  status,2
           goto ksh22
          movlw  0xF0
          andwf freqM,1
          movlw  0x09
          addwf  freqM,1
          ;movlw  0x10
          ;subwf  freqM,1
          movf   freqM,w
          andlw   0xF0
          bcf    status,2
          xorlw   0xF0
          btfss   status,2
          goto   ksh22
           movlw  0x0F
           andwf  freqM,1
           movlw   0x90
           addwf   freqM,1
           decf  freqH,1
           movf  freqH,w
           andlw  0x0F
           bcf    status,2
           xorlw   0x0F
           btfss   status,2
           goto   ksh22           
          movlw  0xF0
          ;movlw  0xF0
          andwf freqH,1
          movlw  0x09
          addwf  freqH,1
        ;  movlw  0x10
         ; subwf  freqH,1          
          
          ;goto   ksh22
          ;decf   freqH,1
ksh22    bcf  status,2           ; detect band switching
             ;movf   divL,w
          movlw  0x5F
          xorwf  divL,w
          btfss   status,2 
         goto   ksh32
          bcf   status,2
           ;movf   divH,w
         movlw  0x0E
          xorwf  divH,w
         btfss   status,2
         goto   ksh42
         bcf   band,1
         bsf   band,0
         bcf   band,2
ksh42     bcf   status,2
          ;movf  divH,w
         movlw  0x22
          xorwf  divH,w
         btfss   status,2
         goto   ksh32
         bcf   band,2
         bcf   band,0
         bsf   band,1
ksh32     bcf status,2  ; to 
          movlw  0x17
          xorwf  divL,w
          btfss  status,2
          goto   ksh72
          bcf  status,2
          movlw   0x06
          xorwf  divH,w
          btfss  status,2
          goto   ksh72
          movlw  0x20
          movwf  divL
          movlw  0x4E
          movwf  divH
          movlw  0x96
          movwf  freqH
          movlw  0x20
          movwf  freqM
          movlw  0x00
          movwf  freqL
ksh72            call   ksh
          call   lcd_out
     btfsc  esp,3
     goto   oprb60
     call pause1
oprb60  call pause
        ;call   pause1
       ;bsf   esp,0
      retlw  00
; ----------------end decrement frequency-----------
;
;---------start weak booster on---------
weakb_on call  iict
  movlw 0x78
  call  act
  movlw 0x00  ; Co=0, RS=1, set instruction register for multiple next data bytes  
  call  act
  movlw 0x30  ; 
  call  act
  movlw 0x0C  ; display config=LSB on top
  call  act
  movlw 0x47  ; address Y=
  call  act
  movlw 0xD0  ; address X=
  call  act
  call  iicp
;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x7E  ; G
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0xB1  ; 
  call  act
  movlw 0x91  ; 
  call  act
  movlw 0x76  ; 
  call  act
   call  iicp
  retlw  00
;-------------end weak booster on---------
;
;-------------start weak booster off------
weakb_off  call  iict
  movlw 0x78
  call  act
  movlw 0x00  ; Co=0, RS=1, set instruction register for multiple next data bytes  
  call  act
  movlw 0x30  ; 
  call  act
  movlw 0x0C  ; display config=LSB on top
  call  act
  movlw 0x47  ; address Y=
  call  act
  movlw 0xD0  ; address X=
  call  act
  call  iicp
   call space1
  retlw  00
;----------------end weak booster off------
; 
;-----------  space  -------------
space1
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  retlw  00
;--------------------------------
;

;  ----- subroutine I2C ks-h-148 ------
acts movwf transo  ; iic data transmission 
;  call  oled  
  movlw 0x08
  movwf count3
vbits  bcf sclks
  call  pause
  btfsc transo,7  
  bsf sdtas
  btfss transo,7
  bcf sdtas
  call  pause
  bsf sclks
  call pause
  bcf sclks
  call pause
  rlf transo,f
  decfsz count3,f
  goto  vbits
  call  ackps
  retlw 00
;***************************************************
iicps  bcf sdtas  ; stop condition module
  call  pause
  bsf sclks
  call  pause
  bsf sdtas  ; stop command
  call  pause
  bcf sclks
  call  pause
  retlw 00
;***************************************************  
iicts  bsf sdtas  ; start condition module 
  bsf sclks
  call  pause
  bcf sdtas  ; start command
  call  pause
  bcf sclks
  call  pause
  retlw 00
;***************************************************
ackps  bsf sclks  ; one clock step for acknowledge (not read)
  call  pause
  bcf sclks
  call  pause
  retlw 00
;

      ;----------- end ---  ks-h-148 ------------
;-------------------------
dig0  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x7E  ; 
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0x7E  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;----------------------------  
dig1  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x82  ; 
  call  act
  movlw 0xFF  ; 
  call  act
  movlw 0x80  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;-------------------------------  
dig2  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x82  ; 
  call  act
  movlw 0xC1  ; 
  call  act
  movlw 0xA1  ; 
  call  act
  movlw 0x91  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x86  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
 ;--------------------------------
dig3  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x43  ; 
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0x81  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x95  ; 
  call  act
  movlw 0x63  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;------------------------------------
dig4  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x30  ; 
  call  act
  movlw 0x28  ; 
  call  act
  movlw 0x24  ; 
  call  act
  movlw 0x22  ; 
  call  act
  movlw 0xFF  ; 
  call  act
  movlw 0x20  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;---------------------------------------
dig5  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x4F  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x71  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;-----------------------------------------
dig6  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x7E  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x72  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;-------------------------------------------
dig7  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x03  ; 
  call  act
  movlw 0x01  ; 
  call  act
  movlw 0xC1  ; 
  call  act
  movlw 0x31  ; 
  call  act
  movlw 0x0D  ; 
  call  act
  movlw 0x03  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;------------------------------------------
dig8  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x76  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x89  ; 
  call  act
  movlw 0x76  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;---------------------------------------------
dig9  ;
   call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x4E  ; 
  call  act
  movlw 0x91  ; 
  call  act
  movlw 0x91  ; 
  call  act
  movlw 0x91  ; 
  call  act
  movlw 0x91  ; 
  call  act
  movlw 0x7E  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  goto  cikl_3
;------------------------------------------------
dot   ;
  call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0xC0  ; 
  call  act
  movlw 0xC0  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x00  ; 
  call  act
   call  iicp
  ;goto  cikl_3
  retlw  00
;-------------------------------------------------

mhz   ;
  call  iict
  movlw 0x78  ; address
  call  act
  movlw 0x40  ; for data
  call  act
  movlw 0xFF  ; 
  call  act
  movlw 0x03  ; 
  call  act
  movlw 0x0C  ; 
  call  act
  movlw 0x30  ; 
  call  act
  movlw 0x0C  ; 
  call  act
  movlw 0x03  ; 
  call  act
  movlw 0xFF  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0xFC  ; 
  call  act
  movlw 0x20  ; 
  call  act
  movlw 0x20  ; 
  call  act
  movlw 0x20  ; 
  call  act
  movlw 0xFC  ; 
  call  act
  movlw 0x00  ; 
  call  act
  movlw 0x84  ; 
  call  act
  movlw 0xC4  ; 
  call  act
  movlw 0xA4  ; 
  call  act
  movlw 0x94  ; 
  call  act
  movlw 0x8C  ; 
  call  act
  ;movlw 0x00  ; 
  ;call  act
   call  iicp
  ;goto  cikl_3
  retlw  00;
;------------- I2C SUBROUTINE --------------------                
act movwf transo  ; iic data transmission 
;  call  oled  
  movlw 0x08
  movwf count3
vbit  bcf sclk
  call  pause
  btfsc transo,7  
  bsf sdta
  btfss transo,7
  bcf sdta
  call  pause
  bsf sclk
  call pause
  bcf sclk
  call pause
  rlf transo,f
  decfsz count3,f
  goto  vbit
  call  ackp
  retlw 00
;***************************************************
iicp  bcf sdta  ; stop condition module
  call  pause
  bsf sclk
  call  pause
  bsf sdta  ; stop command
  call  pause
  bcf sclk
  call  pause
  retlw 00
;***************************************************  
iict  bsf sdta  ; start condition module 
  bsf sclk
  call  pause
  bcf sdta  ; start command
  call  pause
  bcf sclk
  call  pause
  retlw 00
;***************************************************
ackp  bsf sclk  ; one clock step for acknowledge (not read)
  call  pause
  bcf sclk
  call  pause
  retlw 00
;***************************************************
pause movlw 0x01  ; cca. 6ms delay
;      nop
      movwf count1
d1    movlw 0x01    
      movwf count2
d2    decfsz  count2,f  
      goto  d2    
      decfsz  count1,f  
      goto  d1    
      retlw 00
;***************************************************
;
pause1 movlw 0xFF  ; cca. ms delay
;      nop
      movwf count1
d11    movlw 0xFF    
      movwf count2
      movlw  0x10
      movwf count3
d21    decfsz  count2,f  
      goto  d21    
d31    decfsz  count3,f
      goto  d31
      decfsz  count1,f  
      goto  d11
          
      retlw 00

;------------------------------------------------------------
konec
  end
;============================================================
